@@ -331,7 +331,7 @@ def pin_subpackage(metadata, subpackage_name, min_pin='x.x.x.x.x.x', max_pin='x'
 
 
 def native_compiler(language, config):
-    compiler = "unknown_compiler"
+    compiler = language
 
     for platform in [config.platform, config.platform.split('-')[0]]:
         try:
