@@ -831,6 +831,14 @@ class programheader(object):
         if self.p_type == PT_INTERP:
             file.seek(self.p_offset)
             elffile.program_interpreter = file.read(self.p_filesz - 1).decode()
+        elif self.p_type == PT_LOAD:
+            file.seek(self.p_offset)
+            if hasattr(elffile, 'ptload_p_vaddr'):
+                elffile.ptload_p_vaddr.append(self.p_vaddr)
+                elffile.ptload_p_paddr.append(self.p_paddr)
+            else:
+                elffile.ptload_p_vaddr = [self.p_vaddr]
+                elffile.ptload_p_paddr = [self.p_paddr]
 
 
 class elffile(UnixExecutable):
@@ -897,6 +905,10 @@ class elffile(UnixExecutable):
         'Can be called immediately after the elfsections have been constructed'
         for es in self.elfsections:
             if addr >= es.sh_addr and addr < es.sh_addr + es.sh_size:
+                # sections which do not appear in the memory image of the
+                # process should be skipped
+                if es.sh_addr == 0:
+                    continue
                 return es, addr - es.sh_addr
         return None, None
 
@@ -1100,7 +1112,7 @@ def inspect_linkages_otool(filename, arch='native'):
         args.extend(['-arch', os.uname()[4]])
     args.extend(['-L', filename])
     result = check_output(args).decode(encoding='ascii')
-    groups = re.findall('^\t(.*) \(compatibility', result, re.MULTILINE)
+    groups = re.findall(r'^\t(.*) \(compatibility', result, re.MULTILINE)
     return groups
 
 
@@ -1111,7 +1123,7 @@ def inspect_linkages_ldd(filename):
     result, err = process.communicate()
     result = result.decode(encoding='ascii')
     err = err.decode(encoding='ascii')
-    groups = re.findall('^\t(?!linux-gate\.so\.1.*$)[^ ]+ => (.*) \([0-9a-fx]+\)',
+    groups = re.findall(r'^\t(?!linux-gate\.so\.1.*$)[^ ]+ => (.*) \([0-9a-fx]+\)',
                         result, re.MULTILINE)
     return groups
 
@@ -1167,9 +1179,9 @@ def ldd(*args):
 
 def main(argv):
     for idx, progname in enumerate(argv[0:2][::-1]):
-        if re.match('.*ldd(?:$|\.exe|\.py)', progname):
+        if re.match(r'.*ldd(?:$|\.exe|\.py)', progname):
             return ldd(*argv[2 - idx:])
-        elif re.match('.*otool(?:$|\.exe|\.py)', progname):
+        elif re.match(r'.*otool(?:$|\.exe|\.py)', progname):
             return otool(*argv[2 - idx:])
         elif os.path.isfile(progname):
             klass = codefile_class(progname)
