@@ -29,7 +29,7 @@ from conda_build.conda_interface import normalized_version
 from conda_build.conda_interface import human_bytes, hashsum_file
 from conda_build.conda_interface import default_python
 
-from conda_build.utils import tar_xf, unzip, rm_rf, check_call_env, ensure_list
+from conda_build.utils import decompressible_exts, tar_xf, rm_rf, check_call_env, ensure_list
 from conda_build.source import apply_patch
 from conda_build.environ import create_env
 from conda_build.config import Config
@@ -224,7 +224,7 @@ def skeletonize(packages, output_dir=".", version=None, recursive=False,
                 raise RuntimeError("directory already exists: %s" % dir_path)
         d = package_dicts.setdefault(package,
             {
-                'packagename': package.lower(),
+                'packagename': package,
                 'run_depends': '',
                 'build_depends': '',
                 'entry_points': '',
@@ -310,7 +310,7 @@ def skeletonize(packages, output_dir=".", version=None, recursive=False,
 
     for package in package_dicts:
         d = package_dicts[package]
-        name = d['packagename']
+        name = d['packagename'].lower()
         makedirs(join(output_dir, name))
         print("Writing recipe for %s" % package.lower())
         with open(join(output_dir, name, 'meta.yaml'), 'w') as f:
@@ -344,10 +344,10 @@ def skeletonize(packages, output_dir=".", version=None, recursive=False,
             # Always require python as a dependency.  Pip is because we use pip for
             #    the install line.
             ordered_recipe['requirements'] = OrderedDict()
-            ordered_recipe['requirements']['host'] = sorted(['python', 'pip'] +
-                                                            list(set(d['build_depends'])))
-            ordered_recipe['requirements']['run'] = sorted(['python'] +
-                                                           list(set(d['run_depends'])))
+            ordered_recipe['requirements']['host'] = sorted(set(['python', 'pip'] +
+                                                                list(d['build_depends'])))
+            ordered_recipe['requirements']['run'] = sorted(set(['python'] +
+                                                               list(d['run_depends'])))
 
             if d['import_tests']:
                 ordered_recipe['test']['imports'] = d['import_tests']
@@ -383,7 +383,7 @@ def skeletonize(packages, output_dir=".", version=None, recursive=False,
             # Fix the indents
             recipe_lines = []
             for line in rendered_recipe.splitlines():
-                match = re.search('^\s+(-) ', line,
+                match = re.search(r'^\s+(-) ', line,
                                   flags=re.MULTILINE)
                 if match:
                     pre, sep, post = line.partition('-')
@@ -639,18 +639,22 @@ def convert_version(version):
 
 
 MARKER_RE = re.compile(r"(?P<name>^[^=<>!\s]+)"
-                       "\s*"
-                       "(?P<constraint>[=!><]=?\s*[^\s;]+)?"
-                       "(?:\s+;\s+)?(?P<env_mark_name>[^=<>!\s;]+)?"
-                       "\s*"
-                       "(?P<env_mark_constraint>[=<>!\s]+[^=<>!\s]+)?"
+                       r"\s*"
+                       r"(?P<constraint>[=!><]=?\s*[^\s;]+)?"
+                       r"(?:\s+;\s+)?(?P<env_mark_name>[^=<>!\s;]+)?"
+                       r"\s*"
+                       r"(?P<env_mark_constraint>[=<>!\s]+[^=<>!\s]+)?"
                        )
 
 
 def _translate_python_constraint(constraint):
-    operator, value = constraint.split()
-    value = "".join(value.strip("'").strip('"').split(".")[:2])
-    return " ".join((operator, value))
+    parts = constraint.split()
+    translation = constraint
+    if len(parts) == 2:
+        operator, value = parts
+        value = "".join(value.strip("'").strip('"').split(".")[:2])
+        translation = " ".join((operator, value))
+    return translation
 
 
 def env_mark_lookup(env_mark_name, env_mark_constraint):
@@ -857,7 +861,7 @@ def get_package_metadata(package, d, data, output_dir, python_version, all_extra
     else:
         license_name = ' or '.join(licenses)
     # remove the word license from the license
-    clean_license_name = re.subn('(.*)\s+license', r'\1', license_name, flags=re.IGNORECASE)[0]
+    clean_license_name = re.subn(r'(.*)\s+license', r'\1', license_name, flags=re.IGNORECASE)[0]
     d['license'] = clean_license_name
     d['license_family'] = guess_license_family(license_name, allowed_license_families)
     if 'new_hash_value' in pkginfo:
@@ -872,10 +876,8 @@ def valid(name):
 
 
 def unpack(src_path, tempdir):
-    if src_path.endswith(('.tar.gz', '.tar.bz2', '.tgz', '.tar.xz', '.tar')):
+    if src_path.lower().endswith(decompressible_exts):
         tar_xf(src_path, tempdir)
-    elif src_path.endswith('.zip'):
-        unzip(src_path, tempdir)
     else:
         raise Exception("not a valid source: %s" % src_path)
 
