@@ -1,6 +1,6 @@
 from __future__ import absolute_import, division, print_function
 
-from collections import defaultdict
+from collections import OrderedDict, defaultdict
 import contextlib
 import fnmatch
 import hashlib
@@ -21,6 +21,7 @@ import sys
 import shutil
 import tarfile
 import tempfile
+from threading import Thread
 import time
 import yaml
 
@@ -167,6 +168,48 @@ class DummyPsutilProcess(object):
     def children(self, *args, **kwargs):
         return []
 
+def _setup_rewrite_pipe(env):
+    """Rewrite values of env variables back to $ENV in stdout
+
+    Takes output on the pipe and finds any env value
+    and rewrites it as the env key
+
+    Useful for replacing "~/conda/conda-bld/pkg_<date>/_h_place..." with "$PREFIX"
+
+    Returns an FD to be passed to Popen(stdout=...)
+    """
+    # replacements is the env dict reversed,
+    # ordered by the length of the value so that longer replacements
+    # always occur first in case of common prefixes
+    replacements = OrderedDict()
+    for k, v in sorted(env.items(), key=lambda kv: len(kv[1]), reverse=True):
+        replacements[v] = k
+
+    r_fd, w_fd = os.pipe()
+    r = os.fdopen(r_fd, 'rt')
+    if sys.platform == 'win32':
+        replacement_t = '%{}%'
+    else:
+        replacement_t = '${}'
+
+    def rewriter():
+        while True:
+            line = r.readline()
+            if not line:
+                # reading done
+                r.close()
+                os.close(w_fd)
+                return
+            for s, key in replacements.items():
+                line = line.replace(s, replacement_t.format(key))
+            sys.stdout.write(line)
+
+    t = Thread(target=rewriter)
+    t.daemon = True
+    t.start()
+
+    return w_fd
+
 
 class PopenWrapper(object):
     # Small wrapper around subprocess.Popen to allow memory usage monitoring
@@ -289,6 +332,10 @@ def _func_defaulting_env_to_os_environ(func, *popenargs, **kwargs):
     if 'stats' in kwargs:
         del kwargs['stats']
 
+    rewrite_stdout_env = kwargs.pop('rewrite_stdout_env', None)
+    if rewrite_stdout_env:
+        kwargs['stdout'] = _setup_rewrite_pipe(rewrite_stdout_env)
+
     out = None
     if stats is not None:
         proc = PopenWrapper(_args, **kwargs)
@@ -444,7 +491,7 @@ def get_prefix_replacement_paths(src, dst):
     return os.path.join(*ssplit), os.path.join(*dsplit)
 
 
-def copy_into(src, dst, timeout=90, symlinks=False, lock=None, locking=True, clobber=False):
+def copy_into(src, dst, timeout=900, symlinks=False, lock=None, locking=True, clobber=False):
     """Copy all the files and directories in src to the directory dst"""
     log = get_logger(__name__)
     if symlinks and islink(src):
@@ -543,7 +590,7 @@ def copytree(src, dst, symlinks=False, ignore=None, dry_run=False):
     return dst_lst
 
 
-def merge_tree(src, dst, symlinks=False, timeout=90, lock=None, locking=True, clobber=False):
+def merge_tree(src, dst, symlinks=False, timeout=900, lock=None, locking=True, clobber=False):
     """
     Merge src into dst recursively by copying all files from src into dst.
     Return a list of all files copied.
@@ -581,7 +628,7 @@ _lock_folders = (os.path.join(root_dir, 'locks'),
                  os.path.expanduser(os.path.join('~', '.conda_build_locks')))
 
 
-def get_lock(folder, timeout=90):
+def get_lock(folder, timeout=900):
     fl = None
     try:
         location = os.path.abspath(os.path.normpath(folder))
@@ -613,7 +660,7 @@ def get_lock(folder, timeout=90):
     return fl
 
 
-def get_conda_operation_locks(locking=True, bldpkgs_dirs=None, timeout=90):
+def get_conda_operation_locks(locking=True, bldpkgs_dirs=None, timeout=900):
     locks = []
     bldpkgs_dirs = ensure_list(bldpkgs_dirs)
     # locks enabled by default
@@ -950,7 +997,7 @@ def get_skip_message(m):
 def package_has_file(package_path, file_path):
     try:
         locks = get_conda_operation_locks()
-        with try_acquire_locks(locks, timeout=90):
+        with try_acquire_locks(locks, timeout=900):
             with tarfile.open(package_path) as t:
                 try:
                     # internal paths are always forward slashed on all platforms
